@@ -1,5 +1,7 @@
 /*  This file is part of Qwit.
 
+    Copyright (C) 2008, 2009 Artem Iglikov
+    
     Qwit is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation, either version 3 of the License, or
@@ -19,7 +21,10 @@
 #include "MainWindow.h"
 
 #include "QwitException.h"
-
+#include "TwitPicDialog.h"
+#include <QFileDialog>
+#include <QMessageBox>
+#include <QDialog>
 #include <iostream>
 
 using namespace std;
@@ -47,110 +52,124 @@ MainWindow::MainWindow(QWidget *parent): QDialog(parent) {
 	monthes["Nov"] = 11;
 	monthes["Dec"] = 12;
 
-	TwitterWidget *twitterWidget = new TwitterWidget();
-	twitterWidget->setObjectName(QString::fromUtf8("homeTwitterWidget"));
-	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
-
 	QGridLayout *gridLayout = new QGridLayout(homeTab);
 	gridLayout->setObjectName(QString::fromUtf8("homeGridLayout"));
 
 	QScrollArea *scrollArea = new QScrollArea(homeTab);
+
+	TwitterWidget *twitterWidget = new TwitterWidget(scrollArea);
+	twitterWidget->setObjectName(QString::fromUtf8("homeTwitterWidget"));
+	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
+
 	scrollArea->setBackgroundRole(QPalette::Light);
 	scrollArea->setWidget(twitterWidget);
 	scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+	scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
 	gridLayout->addWidget(scrollArea, 0, 0, 1, 1);
 
 	twitterTabs[HOME_TWITTER_TAB] = TwitterTab(scrollArea, twitterWidget, 0);
 
 
-	twitterWidget = new TwitterWidget();
-	twitterWidget->setObjectName(QString::fromUtf8("publicTwitterWidget"));
-	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
-
 	gridLayout = new QGridLayout(publicTab);
 	gridLayout->setObjectName(QString::fromUtf8("publicGridLayout"));
 
 	scrollArea = new QScrollArea(publicTab);
+
+	twitterWidget = new TwitterWidget(scrollArea);
+	twitterWidget->setObjectName(QString::fromUtf8("publicTwitterWidget"));
+	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
+
 	scrollArea->setBackgroundRole(QPalette::Light);
 	scrollArea->setWidget(twitterWidget);
 	scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+	scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
 	gridLayout->addWidget(scrollArea, 0, 0, 1, 1);
 
 	twitterTabs[PUBLIC_TWITTER_TAB] = TwitterTab(scrollArea, twitterWidget, 0);
 
 
-	twitterWidget = new TwitterWidget();
-	twitterWidget->setObjectName(QString::fromUtf8("repliesTwitterWidget"));
-	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
-
 	gridLayout = new QGridLayout(repliesTab);
 	gridLayout->setObjectName(QString::fromUtf8("repliesGridLayout"));
 
 	scrollArea = new QScrollArea(repliesTab);
+
+	twitterWidget = new TwitterWidget(scrollArea);
+	twitterWidget->setObjectName(QString::fromUtf8("repliesTwitterWidget"));
+	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
+
 	scrollArea->setBackgroundRole(QPalette::Light);
 	scrollArea->setWidget(twitterWidget);
 	scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+	scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
 	gridLayout->addWidget(scrollArea, 0, 0, 1, 1);
 
 	twitterTabs[REPLIES_TWITTER_TAB] = TwitterTab(scrollArea, twitterWidget, 0);
 
 
-	twitterWidget = new TwitterWidget();
+	scrollArea = new QScrollArea(customTab);
+
+	twitterWidget = new TwitterWidget(scrollArea);
 	twitterWidget->setObjectName(QString::fromUtf8("customTwitterWidget"));
 	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
 
-	scrollArea = new QScrollArea(customTab);
 	scrollArea->setBackgroundRole(QPalette::Light);
 	scrollArea->setWidget(twitterWidget);
 	scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+	scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
 	vboxLayout1->addWidget(scrollArea);
 
 	twitterTabs[CUSTOM_TWITTER_TAB] = TwitterTab(scrollArea, twitterWidget, 0);
 
-	twitterWidget = new TwitterWidget();
-	twitterWidget->setObjectName(QString::fromUtf8("inboxTwitterWidget"));
-	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
-
 	gridLayout = new QGridLayout(inboxTab);
 	gridLayout->setObjectName(QString::fromUtf8("inboxGridLayout"));
 
 	scrollArea = new QScrollArea(inboxTab);
+
+	twitterWidget = new TwitterWidget(scrollArea);
+	twitterWidget->setObjectName(QString::fromUtf8("inboxTwitterWidget"));
+	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
+
 	scrollArea->setBackgroundRole(QPalette::Light);
 	scrollArea->setWidget(twitterWidget);
 	scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+	scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
 	gridLayout->addWidget(scrollArea, 0, 0, 1, 1);
 
 	twitterTabs[INBOX_TWITTER_TAB] = TwitterTab(scrollArea, twitterWidget, 0);
 
-	twitterWidget = new TwitterWidget();
-	twitterWidget->setObjectName(QString::fromUtf8("outboxTwitterWidget"));
-	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
-
 	gridLayout = new QGridLayout(outboxTab);
 	gridLayout->setObjectName(QString::fromUtf8("outboxGridLayout"));
 
 	scrollArea = new QScrollArea(outboxTab);
+
+	twitterWidget = new TwitterWidget(scrollArea);
+	twitterWidget->setObjectName(QString::fromUtf8("outboxTwitterWidget"));
+	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
+
 	scrollArea->setBackgroundRole(QPalette::Light);
 	scrollArea->setWidget(twitterWidget);
 	scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+	scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
 	gridLayout->addWidget(scrollArea, 0, 0, 1, 1);
 
 	twitterTabs[OUTBOX_TWITTER_TAB] = TwitterTab(scrollArea, twitterWidget, 0);
 
-	twitterWidget = new TwitterWidget();
+	scrollArea = new QScrollArea(searchTab);
+
+	twitterWidget = new TwitterWidget(scrollArea);
 	twitterWidget->setObjectName(QString::fromUtf8("searchTwitterWidget"));
 	twitterWidget->sizePolicy().setHorizontalPolicy(QSizePolicy::Maximum);
 
-	scrollArea = new QScrollArea(searchTab);
 	scrollArea->setBackgroundRole(QPalette::Light);
 	scrollArea->setWidget(twitterWidget);
 	scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+	scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
 	vboxLayout2->addWidget(scrollArea);
 
@@ -165,7 +184,7 @@ MainWindow::MainWindow(QWidget *parent): QDialog(parent) {
 	charsLeftLabel->setText(QString::number(statusTextEdit->getMaxStatusCharacter()));
 	charsLeftLabel->setForegroundRole(QPalette::Light);
 
-	vboxLayout->insertWidget(1, statusTextEdit);
+	horizontalLayout->insertWidget(0, statusTextEdit);
 
 	optionsDialog = new OptionsDialog(this);
 	optionsDialog->setModal(true);
@@ -175,6 +194,7 @@ MainWindow::MainWindow(QWidget *parent): QDialog(parent) {
 	acceptClose = false;
 
 	connect(statusTextEdit, SIGNAL(returnPressed()), this, SLOT(sendStatus()));
+	connect(twitPicButton, SIGNAL(clicked()), this, SLOT(postTwitPic()));
 	connect(customUsernameLineEdit, SIGNAL(returnPressed()), this, SLOT(customUsernameChanged()));
 	connect(searchLineEdit, SIGNAL(returnPressed()), this, SLOT(searchChanged()));
 
@@ -191,10 +211,13 @@ MainWindow::MainWindow(QWidget *parent): QDialog(parent) {
 	for (int i = 0; i < TWITTER_TABS; ++i) {
 		connect(twitterTabs[i].twitterWidget, SIGNAL(reply(const QString &)), statusTextEdit, SLOT(reply(const QString &)));
 		connect(twitterTabs[i].twitterWidget, SIGNAL(replyID(const QString &)), this, SLOT(setReplyID(const QString &)));
-                connect(twitterTabs[i].twitterWidget, SIGNAL(retweet(const QString &, const QString &)), statusTextEdit, SLOT(retweet(const QString &,const QString &)));		connect(statusTextEdit, SIGNAL(getRetweetTag()), this, SLOT(setRetweetTag()));
+		connect(twitterTabs[i].twitterWidget, SIGNAL(retweet(const QString &, const QString &)), statusTextEdit, SLOT(retweet(const QString &,const QString &)));
+		connect(statusTextEdit, SIGNAL(getRetweetTag()), this, SLOT(setRetweetTag()));
 		connect(this, SIGNAL(sendRetweetTag(const QString &, const bool &)), statusTextEdit, SLOT(setRetweetTag(const QString &, const bool &)));
 		connect(twitterTabs[i].twitterWidget, SIGNAL(directMessages(const QString &)), statusTextEdit, SLOT(directMessages(const QString &)));
 		connect(twitterTabs[i].twitterWidget, SIGNAL(directMessagesID(const QString &)), this, SLOT(setDirectMessagesID(const QString &)));
+		connect(twitterTabs[i].twitterWidget, SIGNAL(openUser(const QString &)), this, SLOT(openUser(const QString &)));
+		connect(twitterTabs[i].twitterWidget, SIGNAL(openSearch(const QString &)), this, SLOT(openSearch(const QString &)));
 	}
 
 	connect(statusTextEdit, SIGNAL(leftCharsNumberChanged(int)), this, SLOT(leftCharsNumberChanged(int)));
@@ -213,6 +236,17 @@ MainWindow::MainWindow(QWidget *parent): QDialog(parent) {
 	timer2->start(10000);
 }
 
+void MainWindow::postTwitPic() {
+	QString fileName = QFileDialog::getOpenFileName(this, tr("Pick an image"), QDir::homePath(), tr("Image Files (*.png *.jpg *.jpeg *.bmp)"));
+	if (fileName.isEmpty())
+		return;
+	TwitPicDialog dialog(fileName, this);
+	dialog.setUser(username, password);
+	if (dialog.exec() == QDialog::Accepted) {
+		statusTextEdit->append(dialog.twitPickedUrlString());
+	}
+}
+
 void MainWindow::setupTrayIcon() {
 	showhideAction = new QAction(tr("&Show / Hide"), this);
 	connect(showhideAction, SIGNAL(triggered()), this, SLOT(showhide()));
@@ -225,12 +259,16 @@ void MainWindow::setupTrayIcon() {
 	trayIcon->setContextMenu(trayIconMenu);
 	trayIcon->setIcon(QIcon(":/images/qwit.png"));
 	trayIcon->show();
+	connect(trayIcon, SIGNAL(messageClicked()), this, SLOT(makeActive()));
 	connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
 }
 
 void MainWindow::loadState() {
 	QSettings settings("arti", "qwit");
 
+	QFile file(settings.fileName());
+	file.setPermissions(QFile::ReadUser | QFile::WriteUser);
+
 	settings.beginGroup("MainWindow");
 	resize(settings.value("size", QSize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT)).toSize());
 	move(settings.value("pos", QPoint(DEFAULT_WINDOW_X, DEFAULT_WINDOW_Y)).toPoint());
@@ -243,6 +281,7 @@ void MainWindow::loadState() {
 	messagesPerPage = settings.value("messagesPerPage", DEFAULT_MESSAGES_PER_PAGE).toInt();
 	messagesPerTray = settings.value("messagesPerTray", DEFAULT_MESSAGES_PER_TRAY).toInt();
 	verticalAlignControl = settings.value("verticalAlignControl", true).toBool();
+	placeTabsVertically = settings.value("placeTabsVertically", true).toBool();
 	updatesNotification = settings.value("updatesNotification", true).toBool();
 	usernameUnderAvatar = settings.value("usernameUnderAvatar", true).toBool();
 	twitter.setServiceAPIURL(settings.value("serviceAPIURL", "http://twitter.com").toString());
@@ -271,6 +310,7 @@ void MainWindow::loadState() {
 	optionsDialog->messagesPerTrayLineEdit->setText(QString::number(messagesPerTray));
 	optionsDialog->updatesNotificationCheckBox->setCheckState(updatesNotification ? Qt::Checked : Qt::Unchecked);
 	optionsDialog->verticalControlCheckBox->setCheckState(verticalAlignControl ? Qt::Checked : Qt::Unchecked);
+	optionsDialog->placeTabsVerticallyCheckBox->setCheckState(placeTabsVertically ? Qt::Checked : Qt::Unchecked);
 	optionsDialog->usernameUnderAvatarCheckBox->setCheckState(usernameUnderAvatar ? Qt::Checked : Qt::Unchecked);
 	optionsDialog->serviceBaseURLLineEdit->setText(twitter.getServiceBaseURL());
 	optionsDialog->serviceAPIURLLineEdit->setText(twitter.getServiceAPIURL());
@@ -311,17 +351,18 @@ void MainWindow::loadState() {
 		for (int i = 0; i < size; ++i) {
 			settings.setArrayIndex(i);
 			if (i == 0) {
-				twitterTabs[tab].lastId = settings.value("messageId").toInt();
+				twitterTabs[tab].lastId = settings.value("messageId").toUInt();
 			}
 			twitterTabs[tab].twitterWidget->addItem(
 				settings.value("userpic").toString(),
 				settings.value("username").toString(),
 				settings.value("status").toString(),
 				QDateTime::fromString(settings.value("time").toString(), "yyyy-MM-dd hh:mm:ss"),
-				settings.value("messageId").toInt(),
-				settings.value("replyStatusId").toInt(),
+				settings.value("messageId").toUInt(),
+				settings.value("replyStatusId").toUInt(),
 				-1,
-				twitter.getServiceBaseURL()
+				twitter.getServiceBaseURL(),
+				username
 			);
 		}
 		settings.endArray();
@@ -343,6 +384,18 @@ void MainWindow::setRetweetTag() {
 	emit sendRetweetTag(retweetTag,retweetTagPlace);
 }
 
+void MainWindow::openUser(const QString &user) {
+    tabWidget->setCurrentIndex(CUSTOM_TWITTER_TAB);
+    customUsernameLineEdit->setText(user);
+    customUsernameChanged();
+}
+
+void MainWindow::openSearch(const QString &query) {
+    tabWidget->setCurrentIndex(SEARCH_TWITTER_TAB);
+    searchLineEdit->setText(query);
+    searchChanged();
+}
+
 void MainWindow::sendStatus() {
 	QString status = statusTextEdit->toPlainText().simplified();
 	if (status == "") {
@@ -372,10 +425,14 @@ void MainWindow::updateTimeline() {
 void MainWindow::saveState() {
 	QSettings settings("arti", "qwit");
 
+	QFile file(settings.fileName());
+	file.setPermissions(QFile::ReadUser | QFile::WriteUser);
+
 	for (int tab = 0; tab < TWITTER_TABS; ++tab) {
 		twitterTabs[tab].twitterWidget->setMessagesPerPage(messagesPerPage);
 		twitterTabs[tab].twitterWidget->setUsernameUnderAvatar(usernameUnderAvatar);
 		twitterTabs[tab].twitterWidget->setVerticalAlignControl(verticalAlignControl);
+		twitterTabs[tab].twitterWidget->saveScrollPosition();
 	}
 
 	username = optionsDialog->usernameLineEdit->text();
@@ -396,6 +453,7 @@ void MainWindow::saveState() {
 	retweetTagPlace = optionsDialog->retweetAtEnd->checkState() == Qt::Checked;
 	bool proxySavePassword = optionsDialog->proxySavePasswordCheckBox->checkState() == Qt::Checked;
 	verticalAlignControl = optionsDialog->verticalControlCheckBox->checkState() == Qt::Checked;
+	placeTabsVertically = optionsDialog->placeTabsVerticallyCheckBox->checkState() == Qt::Checked;
 	updatesNotification = optionsDialog->updatesNotificationCheckBox->checkState() == Qt::Checked;
 	usernameUnderAvatar = optionsDialog->usernameUnderAvatarCheckBox->checkState() == Qt::Checked;
 	greetingMessageLabel->setText(optionsDialog->greetingMessageLineEdit->text());
@@ -418,6 +476,7 @@ void MainWindow::saveState() {
 	settings.setValue("retweetTag", retweetTag);
 	settings.setValue("retweetTagPlace", retweetTagPlace);
 	settings.setValue("verticalAlignControl", verticalAlignControl);
+	settings.setValue("placeTabsVertically", placeTabsVertically);
 	settings.setValue("updatesNotification", updatesNotification);
 	settings.setValue("usernameUnderAvatar", usernameUnderAvatar);
 	settings.setValue("serviceBaseURL", twitter.getServiceBaseURL());
@@ -468,8 +527,6 @@ void MainWindow::saveState() {
 	settings.setValue("searchString", searchLineEdit->text());
 	settings.endGroup();
 
-	updateItems();
-
 	if (useProxy) {
 		twitter.useProxy(proxyAddress, proxyPort, proxyUsername, proxyPassword);
 		userpicsDownloader.useProxy(proxyAddress, proxyPort, proxyUsername, proxyPassword);
@@ -483,6 +540,17 @@ void MainWindow::saveState() {
 	} else {
 		statusTextEdit->setDisabled(false);
 	}
+	
+	if (placeTabsVertically) {
+		tabWidget->setTabPosition(QTabWidget::West);
+	} else {
+		tabWidget->setTabPosition(QTabWidget::South);
+	}
+
+	for (int i = 0; i < TWITTER_TABS; ++i) {
+		twitterTabs[i].twitterWidget->resize(twitterTabs[i].scrollArea->width() - twitterTabs[i].scrollArea->verticalScrollBar()->width() - 5, 500);
+		twitterTabs[i].twitterWidget->restoreScrollPosition();
+	}
 }
 
 void MainWindow::resetSettings() {
@@ -572,7 +640,7 @@ void MainWindow::updated(const QByteArray &buffer, int type) {
 	QDomNode node = root.firstChild();
 	QString html = "";
 	QString trayMessage = "";
-	int maxId = twitterTabs[type].lastId;
+	uint maxId = twitterTabs[type].lastId;
 	int j = 0;
 	while (!node.isNull()) {
 		if (node.toElement().tagName() != "status") {
@@ -580,18 +648,18 @@ void MainWindow::updated(const QByteArray &buffer, int type) {
 		}
 		QDomNode node2 = node.firstChild();
 		QString message = "", timeStr = "", user = "", image = "";
-		int id = 0, replyUserID = 0, replyStatusId = 0;
+		uint id = 0, replyUserID = 0, replyStatusId = 0;
 		while (!node2.isNull()) {
 			if (node2.toElement().tagName() == "created_at") {
 				timeStr = node2.toElement().text();
 			} else if (node2.toElement().tagName() == "text") {
 				message = node2.toElement().text();
 			} else if (node2.toElement().tagName() == "id") {
-				id = node2.toElement().text().toInt();
+				id = node2.toElement().text().toUInt();
 			} else if (node2.toElement().tagName() == "in_reply_to_status_id") {
-				replyStatusId = node2.toElement().text().toInt();
+				replyStatusId = node2.toElement().text().toUInt();
 			} else if (node2.toElement().tagName() == "in_reply_to_user_id") {
-				replyUserID = node2.toElement().text().toInt();
+				replyUserID = node2.toElement().text().toUInt();
 			} else if (node2.toElement().tagName() == "user") {
 				QDomNode node3 = node2.firstChild();
 				while (!node3.isNull()) {
@@ -635,12 +703,12 @@ void MainWindow::updated(const QByteArray &buffer, int type) {
 				}
 				imageFileName += (char)c;
 			}
-			imageFileName += "." + QFileInfo(QUrl(image).path()).suffix();
+			//imageFileName += "." + QFileInfo(QUrl(image).path()).suffix();
 			QDir dir(QDir::homePath());
 			dir.mkdir(".qwit");
 			imageFileName = dir.absolutePath() + "/.qwit/" + imageFileName;
 			userpicsDownloader.download(image, imageFileName);
-			twitterTabs[type].twitterWidget->addItem(imageFileName, user, message.simplified(), time.toLocalTime(), id, replyStatusId, j++, twitter.getServiceBaseURL());
+			twitterTabs[type].twitterWidget->addItem(imageFileName, user, message.simplified(), time.toLocalTime(), id, replyStatusId, j++, twitter.getServiceBaseURL(), username);
 		}
 		node = node.nextSibling();
 	}
@@ -658,7 +726,7 @@ void MainWindow::updated(const QByteArray &buffer, int type) {
 	QDomNode node = root.firstChild();
 	QString html = "";
 	QString trayMessage = "";
-	int maxId = twitterTabs[type].lastId;
+	uint maxId = twitterTabs[type].lastId;
 	int j = 0;
 	while (!node.isNull()) {
 		if (node.toElement().tagName() != "direct_message") {
@@ -666,14 +734,14 @@ void MainWindow::updated(const QByteArray &buffer, int type) {
 		}
 		QDomNode node2 = node.firstChild();
 		QString message = "", timeStr = "", user = "", image = "", imageRecipient = "",recipientUser = "";
-		int id = 0;
+		uint id = 0;
 		while (!node2.isNull()) {
 			if (node2.toElement().tagName() == "created_at") {
 				timeStr = node2.toElement().text();
 			} else if (node2.toElement().tagName() == "text") {
 				message = node2.toElement().text();
 			} else if (node2.toElement().tagName() == "id") {
-				id = node2.toElement().text().toInt();
+				id = node2.toElement().text().toUInt();
 			} else if (node2.toElement().tagName() == "sender_screen_name") {
 				user = node2.toElement().text();
 			} else if (node2.toElement().tagName() == "recipient_screen_name") {
@@ -731,12 +799,12 @@ void MainWindow::updated(const QByteArray &buffer, int type) {
 				}
 				imageFileName += (char)c;
 			}
-			imageFileName += "." + QFileInfo(QUrl(image).path()).suffix();
+			//imageFileName += "." + QFileInfo(QUrl(image).path()).suffix();
 			QDir dir(QDir::homePath());
 			dir.mkdir(".qwit");
 			imageFileName = dir.absolutePath() + "/.qwit/" + imageFileName;
 			userpicsDownloader.download(image, imageFileName);
-			twitterTabs[type].twitterWidget->addItem(imageFileName, user, message.simplified(), time.toLocalTime(), id, id, j++, twitter.getServiceBaseURL());
+			twitterTabs[type].twitterWidget->addItem(imageFileName, user, message.simplified(), time.toLocalTime(), id, id, j++, twitter.getServiceBaseURL(), username);
 		}
 		node = node.nextSibling();
 	}
@@ -757,17 +825,17 @@ void MainWindow::updated(const QByteArray &buffer, int type) {
 			QDomNode node = root.firstChild();
 			QString html = "";
 			QString trayMessage = "";
-			int maxId = twitterTabs[type].lastId;
+			uint maxId = twitterTabs[type].lastId;
 			int a = 0;
 			int b = 0;
 			int j = 0;
 			QByteArray buf=buffer;
 			while (buf.indexOf("<entry>", a)!=-1 && buf.indexOf("<entry>", a)<buf.indexOf("</feed>", 0)) {
-				int id = 0;
+				uint id = 0;
 				QString message = "", timeStr = "", user = "", image = "";
 				a=buf.indexOf("<id>", a)+32;
 				b=buf.indexOf("</id>", a);
-				id=buf.mid(a,b-a).toInt();
+				id=buf.mid(a,b-a).toUInt();
 				a=buf.indexOf("<published>", a)+11;
 				b=buf.indexOf("</published>", a);
 				timeStr=buf.mid(a,b-a);
@@ -808,12 +876,12 @@ void MainWindow::updated(const QByteArray &buffer, int type) {
 						}
 						imageFileName += (char)c;
 					}
-					imageFileName += "." + QFileInfo(QUrl(image).path()).suffix();
+					//imageFileName += "." + QFileInfo(QUrl(image).path()).suffix();
 					QDir dir(QDir::homePath());
 					dir.mkdir(".qwit");
 					imageFileName = dir.absolutePath() + "/.qwit/" + imageFileName;
 					userpicsDownloader.download(image, imageFileName);
-					twitterTabs[type].twitterWidget->addItem(imageFileName, user, message.simplified(), time.toLocalTime(), id, id, j++, twitter.getServiceBaseURL());
+					twitterTabs[type].twitterWidget->addItem(imageFileName, user, message.simplified(), time.toLocalTime(), id, id, j++, twitter.getServiceBaseURL(), username);
 				}
 			}
 			twitterTabs[type].lastId = maxId;
@@ -855,6 +923,7 @@ void MainWindow::showhide() {
 		hide();
 	} else {
 		show();
+		activateWindow();
 		for (int i = 0; i < TWITTER_TABS; ++i) {
 			twitterTabs[i].twitterWidget->updateItems();
 		}
@@ -908,4 +977,23 @@ void MainWindow::searchChanged() {
 	updateTimeline();
 }
 
+void MainWindow::keyPressEvent(QKeyEvent *event) {
+	if (event->key() == Qt::Key_Escape) {
+		showhide();
+	} else if ((event->modifiers() == Qt::ControlModifier) && (event->key() == Qt::Key_Q)) {
+		quit();
+	} else {
+		QDialog::keyPressEvent(event);
+	}
+}
+
+void MainWindow::makeActive() {
+	if (isVisible()) {
+		//hide();
+	}
+	show();
+	activateWindow();
+	raise();
+}
+
 #endif
